/* $Id: page.c 4 2023-02-25 09:13:18Z nishi $ */
#include <nikkel/page.h>

#include <nikkel/util.h>
#include <nikkel/config.h>
#include <nikkel/parse.h>
#include <nikkel.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <dirent.h>

void nikkel_page(struct nikkel_config* config, const char* path, FILE* f){
	printf("\r\n");
	printf("<!DOCTYPE html>\n");
	if(f == NULL && !sstartswith(path, ".Category:")){
		printf("<html>\n");
		printf("	<head>\n");
		printf("		<meta charset=\"UTF-8\">\n");
		printf("	</head>\n");
		printf("	<body>\n");
		printf("		404 page was also not found\n");
		printf("	</body>\n");
		printf("</html>\n");
	}else{
		struct nikkel_parsed* page;
		char* category = NULL;
	      	if(f == NULL){
			category = sstrdup(path + sstrlen(".Category:"));
			page = malloc(sizeof(*page));
			page->p_title = sstrcat3("Category \"", category, "\"");
			page->p_body = nikkel_parse_category(config, category);
			page->p_category = malloc(sizeof(*page->p_category));
			page->p_category[0] = NULL;
		}else{
			page = nikkel_parse(config, f);
		}
		printf("<html>\n");
		printf("	<head>\n");
		printf("		<meta charset=\"UTF-8\">\n");
		printf("		<title>%s - %s</title>\n", config->c_name, page->p_title);
		printf("		<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\">\n", config->c_csspath);
		if(config->c_icon != NULL){
			printf("		<link rel=\"icon\" href=\"%s\">\n", config->c_icon);
		}
		printf("	</head>\n");
		printf("	<body>\n");
		printf("		<table border=\"0\" id=\"container\">\n");
		printf("			<tr>\n");
		printf("				<td id=\"navbar\" rowspan=\"2\">\n");
		if(config->c_image != NULL){
			printf("					<div id=\"icon-container\">\n");
			printf("						<a href=\"%s\"><img src=\"%s\" id=\"icon\" alt=\"Wiki Icon\"></a>\n", config->c_prefix, config->c_image);
			printf("					</div>\n");
		}
		printf("					<a href=\"%s/Main_Page\">Main Page</a><br>\n", config->c_prefix);
		printf("					<a href=\"%s/Special:Random_Page\">Random Page</a>\n", config->c_prefix);
		printf("				</td>\n");
		printf("				<td id=\"content\">\n");
		printf("					<h2 id=\"title\">%s</h2>\n", page->p_title);
		printf("					<hr>\n");
		printf("					%s\n", page->p_body);
		if(f != NULL && page->p_category[0] != NULL){
			printf("					<div id=\"bottom\">Category: ");
			int i;
			for(i = 0; page->p_category[i] != NULL; i++){
				printf("<a href=\"%s/Category:%s\">%s</a> ", config->c_prefix, page->p_category[i], page->p_category[i]);
			}
			printf("					</div>\n");
		}
		printf("				</td>\n");
		printf("			</tr>\n");
		printf("			<tr>\n");
		printf("				<td id=\"footer\" rowspan=\"2\">\n");
		printf("					<span id=\"powered\"><i>Powered by Nikkel v%s running on %s</i></span>\n", NIKKEL_VERSION, getenv("SERVER_SOFTWARE"));
		printf("				</td>\n");
		printf("			</tr>\n");
		printf("		</table>\n");
		printf("	</body>\n");
		printf("</html>\n");
		nikkel_parsed_free(page);
		if(category != NULL) free(category);
	}
}

void nikkel_parse_page(struct nikkel_config* config, const char* path){
	if(strcmp(path, ".Special:Random_Page") == 0){
		printf("Status: 303 See Other\r\n");
		char* entry = malloc(1);
		entry[0] = 0;
		DIR* dir = opendir(config->c_wikipath);
		if(dir != NULL){
			struct dirent* d;
			unsigned int counter = 0;
			while((d = readdir(dir)) != NULL){
				if(d->d_name[0] == '.'){
					continue;
				}
				if(d->d_type == DT_REG){
					counter++;
				}
			}
			closedir(dir);
			FILE* f = fopen("/dev/random", "r");
			unsigned int random = 0;
			fread(&random, sizeof(random), 1, f);
			fclose(f);
			unsigned int rnd = counter == 0 ? 0 : (random % counter) + 1;
			dir = opendir(config->c_wikipath);
			counter = 0;
			while((d = readdir(dir)) != NULL){
				if(d->d_name[0] == '.'){
					continue;
				}
				if(d->d_type == DT_REG){
					counter++;
				}
				if(counter == rnd){
					free(entry);
					entry = sstrdup(d->d_name);
					entry[sstrlen(entry) - sstrlen(".wiki")] = 0;
					break;
				}
			}
			closedir(dir);
		
		}
		char* redirect_path = sstrcat3(config->c_prefix, "/", entry);
		printf("Location: %s\r\n\r\nRedirecting ... ", redirect_path);
		free(entry);
		free(redirect_path);
		return;
	}else if(sstartswith(path, ".Category:")){
		nikkel_page(config, path, NULL);
		return;
	}
	FILE* page = fopen(path, "r");
	if(page == NULL){
		printf("Status: 404 Not Found\r\n");
		char* path_404 = nikkel_entry(config, ".404");
		page = fopen(path_404, "r");
		nikkel_page(config, path_404, page);
		free(path_404);
		if(page != NULL) fclose(page);
	}else{
		printf("Status: 200 OK\r\n");
		nikkel_page(config, path, page);
		fclose(page);
	}
}
